import axios, { AxiosError, type AxiosInstance, type AxiosResponse } from 'axios'
import { F_COUNTRY } from '@feedmepos/core/entity'
import { useCoreStore } from '@feedmepos/mf-common'

export const defaultCountry = F_COUNTRY.enum.MY
type ApiVariant = 'portal' | 'auth' | 'localhost'

interface ApiUrl {
  portalBackendUrl: string
  websocketBackendUrl: string
  linkedDeliveryBackend: string
  menuBackendUrl: string
  smsBackendUrl: string
  accountBackendUrl: string
  marketingBackendUrl: string
}

let apiUrl: ApiUrl | null = null

export const SvcConfig = {
  getIdToken(): string | null {
    const idToken = useCoreStore().sessionUser.value?.token
    return idToken ?? null
  },
  getBusinessId(): string | null {
    return useCoreStore().currentBusiness.value?._id ?? null
  },
  getRestaurantId(): string | null {
    return useCoreStore().currentRestaurant.value?._id ?? null
  },
  setRestaurantId(restaurantId: string | null): void {
    const coreStore = useCoreStore()
    if (restaurantId) {
      const restaurant = coreStore.restaurants.value.find((r) => r._id === restaurantId)
      if (restaurant) {
        coreStore.changeRestaurant(restaurant)
      } else {
        console.warn(`Restaurant with ID ${restaurantId} not found.`)
      }
    } else {
      coreStore.changeRestaurant(undefined)
    }
  },
  setBackendUrl() {
    const _country = useCoreStore().currentCountry.value || defaultCountry
    apiUrl = generateBackendUrl(_country)
  },
  getBackendUrl(): ApiUrl {
    const _country = useCoreStore().currentCountry.value || defaultCountry
    return apiUrl || generateBackendUrl(_country)
  }
}

function generateBackendUrl(country: F_COUNTRY): ApiUrl {
  const { namespace } = useCoreStore()
  const isDev = namespace.value === 'dev'
  const area = country !== F_COUNTRY.enum.MY ? `.${country.toLowerCase()}` : ''
  const domain = `${isDev ? '-dev' : ''}${area}.feedmeapi.com`
  const portalBackendUrl = `https://portal-v2${domain}`
  const accountBackendUrl = `https://account${domain}`
  const marketingBackendUrl = `https://portal-marketing${domain}`
  return {
    portalBackendUrl,
    websocketBackendUrl: `wss://94ilf9iqc0.execute-api.ap-southeast-1.amazonaws.com/${isDev ? 'dev' : 'prod'}`,
    linkedDeliveryBackend: `${portalBackendUrl}/linked-deliveries`,
    menuBackendUrl: `${portalBackendUrl}/menu-v4`,
    smsBackendUrl: `https://sms${domain}`,
    accountBackendUrl,
    marketingBackendUrl
  }
}

// eslint-disable-next-line @typescript-eslint/ban-types
interface SvcMasterCompanyResponse<T = {}> {
  status: string
  success: boolean
  data: T
  error: SvsMasterCompanyError
}

interface SvsMasterCompanyError {
  message: string
  code: string
}

class ApiError extends Error {
  constructor(message?: string) {
    super(message)
    this.name = 'ApiError'
    Object.setPrototypeOf(this, new.target.prototype)
  }
}

export class SvcCompanyApiError extends ApiError {
  constructor(message: string) {
    super(message)
    this.name = 'SvcCompanyApiError'
    this.message = message
  }
}

function getBaseUrl(variant: ApiVariant, port?: string): string {
  switch (variant) {
    case 'portal':
      return SvcConfig.getBackendUrl().portalBackendUrl
    case 'auth':
      return SvcConfig.getBackendUrl().accountBackendUrl
    case 'localhost':
      return `http://localhost${port ? `:${port}` : ''}`
  }
}

export const baseClientInstance = (
  path = '',
  variant: ApiVariant = 'portal',
  config?: { noIdRequired?: boolean; url?: string; customIdToken?: string },
  port?: string
): AxiosInstance => {
  const url = config?.url ?? getBaseUrl(variant, port)
  const idToken = SvcConfig.getIdToken()
  if (idToken || config?.noIdRequired || config?.customIdToken) {
    const client = axios.create({
      baseURL: `${url}${path}`,
      headers: {
        Authorization: `Bearer ${config?.customIdToken || idToken}`
      }
    })
    client.interceptors.response.use(
      (response: AxiosResponse<SvcMasterCompanyResponse>) => response,
      async (error): Promise<unknown> => {
        if (error instanceof AxiosError) {
          const data = error.response?.data
          const errMsg = data?.message || data?.error || `${error.message}` || `${error}`
          return Promise.reject(new SvcCompanyApiError(errMsg))
        }
        return Promise.reject(error)
      }
    )
    return client
  } else {
    throw new Error('Unauthorized request')
  }
}

export const authInstance = (config?: {
  noIdRequired?: boolean
  url?: string
  customIdToken?: string
}): AxiosInstance => {
  return baseClientInstance('/', 'auth', config)
}

export const businessClientInstance = (businessId?: string): AxiosInstance => {
  const path = `/businesses/${businessId}`
  if (businessId) {
    return baseClientInstance(path)
  } else {
    throw new Error('Invalid business')
  }
}

export const restaurantClientInstance = (prefix?: string): AxiosInstance => {
  const restaurantId = SvcConfig.getRestaurantId()
  const restaurantPath = `${prefix ?? ''}/restaurants/${restaurantId}`
  if (restaurantId) {
    return baseClientInstance(restaurantPath)
  } else {
    throw new Error('Invalid restaurant')
  }
}

export const menuClientInstance = (): AxiosInstance => {
  const businessId = SvcConfig.getBusinessId()
  if (!businessId) throw new Error('Invalid business ID')
  const { menuBackendUrl } = SvcConfig.getBackendUrl()
  const url = `${menuBackendUrl}/businesses/${businessId}/menu-v4`
  return baseClientInstance('', 'portal', { url })
}

export const smsBackendClientInstance = (): AxiosInstance => {
  const { smsBackendUrl } = SvcConfig.getBackendUrl()
  return baseClientInstance('', 'portal', { url: smsBackendUrl })
}

export const marketingBackendClientInstance = (): AxiosInstance => {
  const { marketingBackendUrl } = SvcConfig.getBackendUrl()
  return baseClientInstance('', 'portal', { url: marketingBackendUrl })
}

export function getData<T>(response: AxiosResponse<T>): T {
  return response.data
}
