import type { AxiosInstance } from 'axios';
import { baseClientInstance, getData, SvcConfig } from '..';
import type { DeliveryCompany, LinkedDeliveryDoc } from '../../views/order-settings/delivery/delivery';
import { FdoFoodpandaSettings, FdoLinkedDelivery, FdoRestaurant, FdoRestaurantFeedmeDelivery, FdoRestaurantInHouseDelivery, FdoShopeeFoodOrder } from '@feedmepos/core/entity';
import type { FdoExternalSetting, FdoFoodpandaOrderOutput, FdoGrabfoodOrderOutput, FdoGrabfoodSettings, FdoShopeeFoodOrderOutput, FdoShopeeFoodSettings, FdtoGetOrderReq } from '@entity';

type DeliveryCompanyUrl = 'foodpanda' | 'grabfood' | 'shopeefood' | 'external'

const deliveryClientInstance = (
  deliveryCompany: DeliveryCompanyUrl
): AxiosInstance => {
  const path = `/${deliveryCompany}`;
  return baseClientInstance(path, 'portal');
};

const linkedDeliveryClientInstance = () => {
  const restaurantId = SvcConfig.getRestaurantId();
  if (!restaurantId) throw new Error('Invalid restaurant ID');
  const { linkedDeliveryBackend } = SvcConfig.getBackendUrl();
  const url = `${linkedDeliveryBackend}/restaurants/${restaurantId}/linked-deliveries`;
  return baseClientInstance('', 'portal', { url });
};


export const remoteOrderApi = {
  async readLinkedDeliveryCompanies(): Promise<LinkedDeliveryDoc[]> {
    return getData(await linkedDeliveryClientInstance().get(''));
  },
  async readLocalDeliveryCompanies(): Promise<DeliveryCompany[]> {
    return getData(await baseClientInstance().get('/deliveries'));
  },
  async createDeliveryCompany(delivery: FdoLinkedDelivery): Promise<LinkedDeliveryDoc> {
    return getData(await linkedDeliveryClientInstance().post('', delivery));
  },
  async updateDeliveryCompany(delivery: LinkedDeliveryDoc): Promise<LinkedDeliveryDoc> {
    return getData(await linkedDeliveryClientInstance().put(`/${delivery._id}`, delivery));
  },
  async deleteDeliveryCompany(delivery: LinkedDeliveryDoc): Promise<LinkedDeliveryDoc> {
    return getData(await linkedDeliveryClientInstance().delete(`/${delivery._id}`));
  },
  integratedDelivery: {
    async updateInhouse(restaurantId: string, dto: FdoRestaurantInHouseDelivery): Promise<FdoRestaurant> {
      return getData(await baseClientInstance().put(`/restaurants/${restaurantId}/in-house-delivery`, dto));
    },
    async updateFeedme(restaurantId: string, dto: FdoRestaurantFeedmeDelivery): Promise<FdoRestaurant> {
      return getData(await baseClientInstance().put(`/restaurants/${restaurantId}/feedme-delivery`, dto));
    },
    async readFoodpanda(restaurantId: string): Promise<FdoFoodpandaSettings | null> {
      return getData(await deliveryClientInstance('foodpanda').get(`/setting/${restaurantId}`));
    },
    async readFoodpandaOrders(req: FdtoGetOrderReq): Promise<FdoFoodpandaOrderOutput[]> {
      return getData(await deliveryClientInstance('foodpanda').get('/order', { params: req }));
    },
    async reflowFoodpandaOrders(orderId: string): Promise<FdoFoodpandaOrderOutput> {
      return getData(await deliveryClientInstance('foodpanda').put(`/order/reflow/${orderId}`));
    },
    async rejectFoodpandaOrders(orderId: string): Promise<FdoFoodpandaOrderOutput> {
      return getData(await deliveryClientInstance('foodpanda').put(`/order/void/${orderId}`));
    },
    async updateFoodpanda(restaurantId: string, dto: FdoFoodpandaSettings): Promise<FdoFoodpandaSettings> {
      return getData(await deliveryClientInstance('foodpanda').put(`/setting/${restaurantId}`, {
        _id: restaurantId,
        ...dto,
      }));
    },
    async syncFoodpandaMenu(restaurantId: string): Promise<FdoFoodpandaSettings> {
      return getData(await deliveryClientInstance('foodpanda').put(`/menu-sync/${restaurantId}`));
    },
    async readGrabfood(restaurantId: string): Promise<FdoGrabfoodSettings | null> {
      return getData(await deliveryClientInstance('grabfood').get(`/setting/${restaurantId}`));
    },
    async readGrabfoodOrders(req: FdtoGetOrderReq): Promise<FdoGrabfoodOrderOutput[]> {
      return getData(await deliveryClientInstance('grabfood').get('/order', { params: req }));
    },
    async reflowGrabfoodOrders(orderId: string): Promise<FdoGrabfoodOrderOutput> {
      return getData(await deliveryClientInstance('grabfood').put(`/order/reflow/${orderId}`));
    },
    async rejectGrabfoodOrders(orderId: string): Promise<FdoGrabfoodOrderOutput> {
      return getData(await deliveryClientInstance('grabfood').put(`/order/void/${orderId}`));
    },

    async updateGrabfood(restaurantId: string, dto: FdoGrabfoodSettings): Promise<FdoRestaurant> {
      return getData(await deliveryClientInstance('grabfood').put(`/setting/${restaurantId}`, {
        _id: restaurantId,
        ...dto,
      }));
    },
    async syncGrabfoodMenu(merchantId: string): Promise<FdoGrabfoodSettings> {
      return getData(await deliveryClientInstance('grabfood').get(`/menu-sync/${merchantId}`));
    },
    async startSelfServeFlow(restaurantId: string): Promise<string> {
      return getData(await deliveryClientInstance('grabfood').get(`/setting/integration-url/${restaurantId}`));
    },

    async readShopeefood(restaurantId: string): Promise<FdoShopeeFoodSettings | null> {
      return getData(await deliveryClientInstance('shopeefood').get(`/setting/${restaurantId}`));
    },
    async readShopeefoodOrders(req: FdtoGetOrderReq): Promise<(FdoShopeeFoodOrderOutput & FdoShopeeFoodOrder)[]> {
      return getData(await deliveryClientInstance('shopeefood').get('/order', { params: req }));
    },
    async reflowShopeefoodOrders(orderId: string): Promise<FdoShopeeFoodOrderOutput> {
      return getData(await deliveryClientInstance('shopeefood').put(`/order/reflow/${orderId}`));
    },
    async rejectShopeefoodOrders(orderId: string): Promise<FdoShopeeFoodOrderOutput> {
      return getData(await deliveryClientInstance('shopeefood').put(`/order/void/${orderId}`));
    },
    async updateShopeefood(restaurantId: string, dto: FdoShopeeFoodSettings): Promise<FdoRestaurant> {
      return getData(await deliveryClientInstance('shopeefood').put(`/setting/${restaurantId}`, {
        _id: restaurantId,
        ...dto,
      }));
    },
    async syncShopeefoodMenu(storeId: string): Promise<FdoShopeeFoodSettings> {
      return getData(await deliveryClientInstance('shopeefood').get(`/menu-sync/${storeId}`));
    },
    async readExternal(restaurantId: string): Promise<FdoExternalSetting | null> {
      return getData(await deliveryClientInstance('external').get(`/setting/${restaurantId}`))
    },
    async updateExternal(restaurantId: string, dto: FdoExternalSetting): Promise<FdoExternalSetting> {
      return getData(await deliveryClientInstance('external').put(`setting/${restaurantId}`, {
        ...dto
      }))
    },
    async syncExternalMenu(restaurantId: string): Promise<FdoExternalSetting> {
      return getData(await deliveryClientInstance('external').get(`/menu-sync/${restaurantId}`));
    },
    async flowTestOrder(
      restaurantId: string,
      platform: DeliveryCompanyUrl
    ): Promise<void> {
      return getData(
        await deliveryClientInstance(platform).put(`/test-order/${restaurantId}`)
      );
    },
    async cancelTestOrder(
      restaurantId: string,
      platform: DeliveryCompanyUrl
    ): Promise<void> {
      return getData(
        await deliveryClientInstance(platform).put(
          `/test-order/cancel/${restaurantId}`,
        )
      );
    },
    async reflowFpOrderNotInDb(payload: { restaurantId: string; slot: string }): Promise<void> {
      return getData(
        await deliveryClientInstance('foodpanda').put('/order/reconcile', payload)
      );
    },
    async reflowGfOrderNotInDb(payload: { restaurantId: string; slot: string; date: string }): Promise<void> {
      return getData(
        await deliveryClientInstance('grabfood').put('/order/reconcile', payload)
      );
    }
  },
};