import { ref } from 'vue';
import { useSnackbarFunctions } from '@/components/snackbar'

export const useLoading = () => {
  const { showError } = useSnackbarFunctions()

  const isLoading = ref(false);

  function startLoading() {
    isLoading.value = true;
  }

  function stopLoading() {
    isLoading.value = false;
  }

  /**
   * A more advance way that automatically calls startLoading() and stopLoading() based on callback
   * @param cb Callback that performs asynchronous task.
   * @returns Returns generic result based on passed callback.
   */
  async function startAsyncCall<T>(cb: () => Promise<T>): Promise<T> {
    try {
      startLoading();
      return await cb();
    } finally {
      stopLoading();
    }
  }

  /**
   * A more advance way that automatically calls startLoading() and stopLoading() based on callback
   * @param cb Callback that performs asynchronous task.
   * @returns Returns generic result based on passed callback. Throw error if have
   */
  async function startAsyncCallWithErr<T>(cb: () => Promise<T>): Promise<T> {
    try {
      startLoading();
      return await cb();
    } catch (e) {
      showError(`${e}`)
      throw e;
    } finally {
      stopLoading();
    }
  }

  return {
    isLoading,
    startLoading,
    stopLoading,
    startAsyncCall,
    startAsyncCallWithErr
  };
};