/// <reference types="googlemaps" />
import { type MapCoordinate } from '@/components/type';
import { ref } from "vue"

const _googleMap = ref<typeof google | null>(null)

// defines a getter
const googleMap = (): typeof google => {
  return _googleMap.value!
}

export function setGoogleMap(map: typeof google) {
  if (!_googleMap.value) _googleMap.value = map
}

const coordinates = Object.freeze({
  'jb': {
    lat: 1.492659,
    lng: 103.7413591
  },
});

function coordinateToArray(v: { lat: number; lng: number }) {
  return [v.lng, v.lat];
}

function arrayToCoordinate(arr: number[]) {
  if (arr.length !== 2) throw Error('Invalid coordinate array');
  return {
    lat: arr[1],
    lng: arr[0]
  };
}

function generateAreaPaths({
  center,
  radius
}: {
  radius: number
  center: {
    lng: number
    lat: number
  }
}): { lat: number; lng: number }[] {
  const searchPoint: google.maps.LatLng[] = []
  const radiusLat = (radius / 6371) * (180 / Math.PI)
  const radiusLng = radiusLat / Math.cos(center.lat * (Math.PI / 180))
  for (let a = 0; a < 361; a += 45) {
    const aRad = a * (Math.PI / 180)
    const x = center.lng + radiusLng * Math.cos(aRad)
    const y = center.lat + radiusLat * Math.sin(aRad)
    const map = googleMap()
    const point = new map.maps.LatLng(y, x, true)
    searchPoint.push(point)
  }
  return searchPoint.map((v) => ({ lat: v.lat(), lng: v.lng() }))
}

function pathToPolygon(path: MapCoordinate[]): number[][] {
  return path.map((p) => coordinateToArray(p));
}

function polygonToPath(polygon: number[][]): MapCoordinate[] {
  return polygon.map((vertex) => arrayToCoordinate(vertex));
}

function getPolygonCenter(polygon: number[][]): MapCoordinate | undefined {
  if (!polygon.length) return undefined;

  const { lat, lng } = arrayToCoordinate(polygon[0]);
  const { maxLng, minLng, maxLat, minLat } = polygon.reduce((pv, path) => {
    const { lng, lat } = arrayToCoordinate(path);
    if (lng > pv.maxLng) pv.maxLng = lng;
    if (lng < pv.minLng) pv.minLng = lng;
    if (lat > pv.maxLat) pv.maxLat = lat;
    if (lat < pv.minLat) pv.minLat = lat;
    return pv;
  }, {
    maxLng: lng,
    minLng: lng,
    maxLat: lat,
    minLat: lat
  });

  return {
    lng: minLng + ((maxLng - minLng) / 2),
    lat: minLat + ((maxLat - minLat) / 2),
  };
}

export {
  googleMap,
  coordinates,
  coordinateToArray,
  arrayToCoordinate,
  generateAreaPaths,
  pathToPolygon,
  polygonToPath,
  getPolygonCenter,
};