import { fullVariantCombination } from '@feedmepos/menu';
import { FdoItemOverriderV4, FdoMenuV4Item, FdoItemVariant, FdoMenuV4VariantGroup, FdoProductTaxSetting, FdtoV4Tax } from '@feedmepos/menu/entity';
import { Country, Dinero } from '@feedmepos/core';
import { useAppStore } from '@/stores/app'

import { useMenuStore } from '@/stores/menu/menu';
import type { CheckboxOption, } from '@/components/type';

export interface Option {
  label: string;
  value: string | null;
}

export type Item = Omit<FdoMenuV4Item, 'variant'> & {
  codeName: string;
  variant?: Omit<FdoItemVariant, 'variantCombinations'> & {
    variantCombinations: (VariantItem & { codeName: string })[]
  }
};
export interface CategorizedItem {
  id: string | null;
  name: string;
  items: Item[];
}

interface VariantItem extends FdoItemOverriderV4 {
  codeName: string;
}


export function mapVariant(
  variantGroupIds: string[] = []
): { [key: string]: string } {
  const variants = variantGroupIds.map((id) => useMenuStore().variants.find((v) => v._id === id)).filter((v) => !!v) as FdoMenuV4VariantGroup[];
  const combinations = fullVariantCombination(variants);
  return combinations.reduce<{ [key: string]: string }>((pv, c) => {
    pv[c.combinationKey] = c.selectionName.join(' / ');
    return pv;
  }, {});
}

export function updateBaseItemOverrider(
  itemId: string,
  items: FdoItemOverriderV4[],
  updateFn: (i: FdoItemOverriderV4) => FdoItemOverriderV4,
): FdoItemOverriderV4[] {
  const base = items.find((i) => !i.combinationKey) ?? { _id: itemId };
  const variants = items.filter((i) => !!i.combinationKey);
  return [updateFn(base), ...variants];
}

export interface ProductOption extends CheckboxOption {
  variant?: CheckboxOption[];
}

export interface Category {
  id: string;
  name: string;
  items: ProductOption[];
}


export type Tax = FdtoV4Tax[] | null | undefined;
export class ItemTax {
  static encodeTax(tax: FdoProductTaxSetting) {
    return `${tax.systemCode}/${tax.taxCode}`;
  }

  static decodeTax(identifier: string): FdoProductTaxSetting {
    const [systemCode, taxCode] = identifier.split('/');
    return { systemCode, taxCode };
  }

  static taxSetting() {
    const countryFlattenTaxes = Country.getCountryFlattenTaxes(useAppStore().country.code);
    return {
      systemCode: [...new Set(countryFlattenTaxes.map((tax) => tax.systemCode))].join('/'),
      countryTaxOptions: countryFlattenTaxes.map((tax) => ({
        label: `${tax.name} (${Dinero.fromFdoDecimal({
          decimal: tax.rate
        }).multiply(100).toFormat(false)}%)`,
        value: ItemTax.encodeTax(tax),
      })),
    };
  }

  static getSelectedTax(tax: Tax, fallbackTax: Tax, taxOptions: { label: string, value: string }[]): string | undefined {
    return (tax ?? fallbackTax ?? [])
      .map((t) => ItemTax.encodeTax(t))
      .find((tax) => taxOptions.some((opt) => opt.value === tax));
  }
}
