import './assets/main.css'

import { computed, createApp } from 'vue'
import { createPinia } from 'pinia'
import Entry from './Entry.vue'
import { router } from './router'
import { initialize, useCoreStore } from '@feedmepos/mf-common'
import { FeedMeUI } from '@feedmepos/ui-library';
import { AuthApi } from './api/auth'
import { getAuth, signInWithCustomToken } from 'firebase/auth'
import { initializeApp } from 'firebase/app'
import { useIframeStore } from './stores/iframe'
import { createFeedMeI18n, type I18n, detectLocale } from "@feedmepos/mf-common"
import enUS from "./locales/en-US.json"
import zhCN from "./locales/zh-CN.json"
import thTH from "./locales/th-TH.json"

export const i18nMessages = {
    "en-US": enUS,
    "zh-CN": zhCN,
    "th-TH": thTH,
}

function initI18n(): I18n {
    const i18n = createFeedMeI18n({
        locale: detectLocale(),
        messages: i18nMessages,
    })
    return i18n
}


const app = createApp(Entry)

app.use(FeedMeUI, {
    locale: detectLocale(),
})

app.use(createPinia())
app.use(router)
app.use(initI18n())


const CoreStore = useCoreStore();
const iframeMode = computed(() => {
    return new URLSearchParams(window.location.search).get('iframe') === 'true';
});

useIframeStore().init(iframeMode.value);

const firebaseApp = initializeApp({
    apiKey: import.meta.env.VITE_FIREBASE_API_KEY,
    authDomain: import.meta.env.VITE_FIREBASE_AUTH_DOMAIN,
    projectId: import.meta.env.VITE_FIREBASE_PROJECT_ID,
    storageBucket: import.meta.env.VITE_FIREBASE_STORAGE_BUCKET,
    messagingSenderId: import.meta.env.VITE_FIREBASE_MESSAGING_SENDER_ID,
    appId: import.meta.env.VITE_FIREBASE_APP_ID,
    measurementId: import.meta.env.VITE_FIREBASE_MEASUREMENT_ID,
    databaseURL: import.meta.env.VITE_FIREBASE_DATABASE_URL,
});

const firebaseAuth = getAuth(firebaseApp);

async function signInWithToken(token: string) {
    const customToken = await AuthApi.getCustomToken(token);
    const { user } = await signInWithCustomToken(firebaseAuth, customToken);
    await CoreStore.signInWithToken({ ...user, id: user.uid }, await user.getIdToken());
}

async function init() {
    if (iframeMode.value) {
        router.beforeEach((to, from, next) => {
            window.addEventListener('message', async (event: MessageEvent<{ event: string, data: any }>) => {
                switch (event.data.event) {
                    case 'init':
                        await signInWithToken(event.data.data.token);
                        await CoreStore.changeCountry(event.data.data.country);
                        await CoreStore.changeBusiness(event.data.data.business);
                        CoreStore.changeRestaurant(event.data.data.restaurant);
                        break;
                    default:
                        break;
                }
            });
            next();
        });
    }
    if (!iframeMode.value || import.meta.env.VITE_DEBUG_MODE === 'true') {
        initialize(import.meta.env.VITE_AUTH_CLIENT_ID);
        CoreStore.setNamespace(import.meta.env.DEV ? 'dev' : 'prod');
    }
}


init().then(() => app.mount('#app'))