import { generator, overrideMenu } from '@feedmepos/menu';
import { F_MENU_VERSION, FdoHasPosition, FdoItemOverriderV4, FdoItemVariant, FdoMenuV4, FdoMenuV4Catalog, FdoMenuV4Category, FdoMenuV4Item, FdoMenuV4VariantGroup, FdtoRestaurantWithMenuV4 } from '@feedmepos/menu/entity';

import { defineStore } from 'pinia';
import { menuApi } from '@/api/menu';
import { sort } from '@/helpers/iteration';
import type { BaseRules, MenuRules } from '../type';
import { mapVariant, type CategorizedItem } from '@/helpers/menu';
import { useCoreStore } from '@feedmepos/mf-common';

interface MenuState {
  restaurants: FdtoRestaurantWithMenuV4[];
  menu: { _rev: string } & FdoMenuV4;
  currentCatalogId: string | null;
  currentSchedulerId: string;
}

function initMenuState(): MenuState {
  return {
    restaurants: [],
    menu: { ...generator.initMenuV4(), _rev: '' },
    currentCatalogId: null,
    currentSchedulerId: '',
  };
}

export function sortPosition<T extends FdoHasPosition>(
  arr: T[],
  elseIf: keyof T | ((v: T) => string),
) {
  const cloned = arr;
  const result = cloned.reduce<{ hasPosition: T[], noPosition: T[] }>((pv, v) => {
    if (v.position == null) {
      pv.noPosition.push(v);
    } else {
      pv.hasPosition.push(v);
    }

    return pv;
  }, { hasPosition: [], noPosition: [] });

  return [
    ...result.hasPosition.sort((a, b) => (a.position as number) - (b.position as number)),
    ...sort(result.noPosition, { selector: elseIf }),
  ];
}
type Item = Omit<FdoMenuV4Item, 'variant'> & {
  codeName: string;
  variant?: Omit<FdoItemVariant, 'variantCombinations'> & {
    variantCombinations: (VariantItem & { codeName: string })[]
  }
};

interface VariantItem extends FdoItemOverriderV4 {
  codeName: string;
}


export function categorizedItems(
  categories: (Omit<FdoMenuV4Category, '_id'> & { _id: string | null })[],
  items: FdoMenuV4Item[],
): CategorizedItem[] {
  const mapped = items.reduce<{ [key: string]: Item[] }>((pv, p) => {
    const key = p.category || '';
    const combinations = mapVariant(p.variant?.variantGroupIds);
    const i = {
      ...p,
      codeName: [p.code, p.name].filter(v => !!v).join(' '),
      variant: {
        variantGroupIds: p.variant?.variantGroupIds || [],
        variantCombinations: (p.variant?.variantCombinations || []).map(vc => {
          const code = vc.code;
          const name = vc.name || combinations[vc.combinationKey || ''];
          return {
            ...vc,
            codeName: [code, name].filter(v => !!v).join(' ').trim() || '-',
          };
        })
      },
    };
    pv[key] = !pv[key] ? [i] : [...pv[key], i];
    return pv;
  }, {});

  return categories.map((category) => {
    return {
      id: category._id,
      name: category.name,
      items: mapped[category._id || ''] || [],
    };
  });
}

export const useMenuStore = defineStore('menu', {
  state: () => {
    return initMenuState();
  },
  getters: {
    version() {
      return (useCoreStore().currentBusiness?.value?.menuVersion || F_MENU_VERSION.enum.v3) as F_MENU_VERSION;
    },
   
    mergedMenu(): FdoMenuV4 {
      const res = overrideMenu({
        menu: this.menu as FdoMenuV4,
        catalogId: this.currentCatalogId,
        
      });

      
      return res;

    },
    catalogs(): FdoMenuV4Catalog[] {
      const catalogs = this.mergedMenu.modules.catalog;
      return catalogs
    },
    currentCatalog: (state): FdoMenuV4Catalog | undefined => {
      return state.menu.modules.catalog.find(
        (catalog) => catalog._id === state.currentCatalogId
      );
    },

    catalogOptions(): { label: string; value: string | null }[] {
      return [
        {
          label: 'Main',
          value: null,
        },
        ...this.catalogs.map((catalog) => ({
          label: catalog.name,
          value: catalog._id,
        })),
      ];
    },
    categories(): (Omit<FdoMenuV4Category, '_id'> & { _id: string | null })[] {
      const categories = this.mergedMenu.modules.category;
      return [
        ...this.rules.item.addon ? [{ _id: this.currentCatalogId ? '' : null, name: 'Addon', position: 0 }] : [],
        ...sortPosition(categories, (category) => category.name)
      ];
    },
    items(): FdoMenuV4Item[] {
      const items = this.mergedMenu.modules.item;
      return sortPosition(items, (item) => item.code || item.name);
    },
    sortedItems(): { [key: string]: number } {
      const category = this.menu.modules.category;
      const sorted = sortPosition(category, (c) => c.name)
        .reduce<{ [key: string]: number }>((pv, c, index) => {
          pv[c._id] = index;
          return pv;
        }, { ['']: 0 });

      const items = this.menu.modules.item;
      return sortPosition(items, (item) => item.code || item.name)
        .reduce<{ [key: string]: number }>((pv, item, index) => {
          pv[item._id] = index + sorted[item.category || ''];
          return pv;
        }, {});

    },
    variants(): FdoMenuV4VariantGroup[] {
      const variants = this.mergedMenu.modules.variant;
      return sort(variants, { selector: 'name' });
    },
    categorizedItems(): CategorizedItem[] {
      return categorizedItems(this.categories, this.items);
    },
    rules(): MenuRules {
      const isDefault = this.currentCatalogId == undefined;
      const isV4 = this.version === F_MENU_VERSION.enum.v4;
      const noSchedule = this.currentSchedulerId === '';
      const baseRules: BaseRules = {
        create: isDefault && noSchedule,
        edit: isDefault && noSchedule,
        delete: isDefault && noSchedule,
      };
      const allItems = this.items.map(v => v._id);
      const markupIds = this.currentCatalog?.markupOptions?.reduce<string[]>((pv, opt) => {
        pv = pv.concat(opt.includeItems == null ? allItems : opt.includeItems.map(v => v._id));
        return pv;
      }, []);

      return {
        item: {
          ...baseRules,
          edit: isV4 || noSchedule,
          sort: isV4 && isDefault && noSchedule,
          addon: isV4,
          code: noSchedule,
          name: isDefault && noSchedule,
          description: isDefault && noSchedule,
          category: (isV4 || isDefault) && noSchedule,
          thumbnail: isDefault && noSchedule,
          orderFrom: (isV4 || isDefault) && noSchedule,
          orderFromNone: isDefault,
          orderFromActiveOnly: !isV4,
          openPrice: isDefault && noSchedule,
          unit: isDefault && noSchedule,
          price: isDefault,
          editPrice: isDefault,
          minPrice: isV4 && noSchedule,
          costing: isV4 && isDefault && noSchedule,
          tax: isDefault && noSchedule,
          variant: isDefault && noSchedule,
          maxVariant: !isV4 ? 2 : 4,
          group: isDefault && noSchedule,
          overrideProductGroup: isV4 && noSchedule,
          inventory: (isV4 || isDefault) && noSchedule,
          bindToVariant: isV4,
          print: isDefault && noSchedule,
          toRoot: isV4 && noSchedule,
          takeaway: (category) => !!category && (isV4 || isDefault) && noSchedule,
          multiScheduler: isV4,
          canRearrange: isV4,
          sequentialKds: isV4,
        },
        category: {
          ...baseRules,
          sort: isDefault,
        },
        group: {
          ...baseRules,
          edit: isV4 || noSchedule,
          sort: isDefault,
          applyProduct: isDefault,
          showAppliedProduct: noSchedule,
          type: !isV4,
          insertRawItems: !isV4
        },
        scheduler: {
          override: isV4,
        },
        takeaway: {
          ...baseRules,
          edit: (isV4 || isDefault) && noSchedule,
          name: isDefault,
        },
        catalog: {
          name: !isDefault,
          defaultOrderFrom: isV4,
          markup: isV4,
        }
      };
    }
  },
  actions: {
    async readMenu() {
      this.menu = await menuApi.readMenu();
      this.selectCatalog(null);
    },
    selectCatalog(id: string | null) {
      this.currentCatalogId = id;
    },
  }
  
});
