import { defineStore } from 'pinia'
import { restaurantApi, type RestaurantSettingDoc } from '@/api/restaurant-setting'
import {
  F_FEATURE,
  F_PAYOUT_ACCOUNT_STATUS,
  FdoFoodpandaSettings,
  FdoRestaurantDineIn,
  FdoRestaurantFeedmeDelivery,
  FdoRestaurantInHouseDelivery,
  FdoRestaurantPickup,
  FdoServiceChargeSetting,
  FdoShopeeFoodSettings,
  type FdoAddress,
  type FdoProfile,
  FdoRestaurant,
  FdoRestaurantSetting,
  type FdoEffectGroup
} from '@feedmepos/core/entity'
import { FdoExternalSetting, FdoGrabfoodSettings } from '@entity'
import { FeatureFlag, Pos } from '@feedmepos/core'
import { SvcConfig } from '@/api'
import { remoteOrderApi } from '@/api/remoteOrder'
import { useI18n, useCoreStore} from '@feedmepos/mf-common'
import { computed, ref, watch} from 'vue'


export interface Profile extends FdoProfile {
  address: Address
}

export interface Address extends FdoAddress {
  formatted: string
  isValidPostcode: boolean
  isValid: boolean
  isValidCoordinate: boolean
}

export interface ExtendedRestaurant extends FdoRestaurant {
  id: string
  profile: Profile
  allowEPayment: true | string
  msicCode?: string
  tinNo?: string
  isEInvoiceActive?: boolean
}

interface RestaurantState {
  restaurantSettings: { [restaurantId: string]: RestaurantSettingDoc }
}

export declare function setRestaurantId(id?: string | null): void

function initRestaurantState(): RestaurantState {
  return {
    restaurantSettings: {}
  }
}

export const useRestaurantStore = defineStore('restaurant', () =>  {
  const CoreStore = useCoreStore()
  const state = ref(initRestaurantState())
  const currentRestaurantSetting = computed<RestaurantSettingDoc| undefined>(() => state.value.restaurantSettings[CoreStore.currentRestaurant.value?._id || ''])

  async function readRestaurantSetting(restaurantId: string) {
    const restaurantSetting = await restaurantApi.readRestaurantSetting(restaurantId)
    state.value.restaurantSettings = {
      ...state.value.restaurantSettings,
      [restaurantId]: restaurantSetting
    }
    return restaurantSetting
  }

  async function selectRestaurant(restaurantId: string) {
    await readRestaurantSetting(restaurantId)
    SvcConfig.setRestaurantId(restaurantId)
    setRestaurantId(restaurantId)
    FeatureFlag.setRestaurant(restaurantId || undefined)
    localStorage.set('feedme_restaurant', restaurantId)
  }

  async function updateServiceCharge(serviceCharge: FdoServiceChargeSetting) {
    await restaurantApi.updateServiceCharge(serviceCharge)
  }
  async function updateDineInSetting(setting: FdoRestaurantDineIn) {
    await restaurantApi.updateDineInSetting(setting)
  }
  async function updatePickupSetting(setting: FdoRestaurantPickup) {
    await restaurantApi.updatePickupSetting(setting)
  }

  async function updateInhouseDelivery(restaurantId: string, dto: FdoRestaurantInHouseDelivery) {
    await remoteOrderApi.integratedDelivery.updateInhouse(restaurantId, dto)
  }
  async function updateFeedmeDelivery(restaurantId: string, dto: FdoRestaurantFeedmeDelivery) {
    return await remoteOrderApi.integratedDelivery.updateFeedme(restaurantId, dto)
  }
  async function updateIntegratedFoodpandaDelivery(restaurantId: string, dto: FdoFoodpandaSettings) {
    await remoteOrderApi.integratedDelivery.updateFoodpanda(restaurantId, dto)
  }
  async function updateIntegratedGrabfoodDelivery(restaurantId: string, dto: FdoGrabfoodSettings) {
    await remoteOrderApi.integratedDelivery.updateGrabfood(restaurantId, dto)
  }
  async function updateIntegratedShopeefoodDelivery(restaurantId: string, dto: FdoShopeeFoodSettings) {
    await remoteOrderApi.integratedDelivery.updateShopeefood(restaurantId, dto)
  }
  async function updateExternalDelivery(restaurantId: string, dto: FdoExternalSetting) {
    await remoteOrderApi.integratedDelivery.updateExternal(restaurantId, dto)
  }
  
  function getRestaurantEPayment(): true | string {
    const { t } = useI18n()
    if (!CoreStore.currentRestaurant.value) {
      return 'getRestaurantEPayment - Invalid restaurant ';
    }
    const payoutAccount = CoreStore.currentRestaurant.value.payoutAccount;
    const posVersion = CoreStore.currentRestaurant.value.posVersion;

    const validPosVersion =
      !!posVersion &&
      Pos.canUse({
        feature: F_FEATURE.enum.qrEPayment,
        posVersion
      })

    if (!validPosVersion)
      return t('order.payoutPosVersion', { version: Pos.minVersionToUse(F_FEATURE.enum.qrEPayment) })
    if (!payoutAccount || !payoutAccount.enable) {
      return t('order.payoutAccountIsRequired')
    }
    if (payoutAccount.status !== F_PAYOUT_ACCOUNT_STATUS.enum.APPROVED) {
      return t('order.payoutAccountIsRequired')
    }
    return true
  }

  return {
    state: state.value,
    currentRestaurantSetting : currentRestaurantSetting.value,
    readRestaurantSetting,
    selectRestaurant,
    updateServiceCharge,
    updateDineInSetting,
    updatePickupSetting,
    updateInhouseDelivery,
    updateFeedmeDelivery,
    updateIntegratedFoodpandaDelivery,
    updateIntegratedGrabfoodDelivery,
    updateIntegratedShopeefoodDelivery,
    updateExternalDelivery,
    getRestaurantEPayment,
  }
})
