import { restaurantApi } from '@/api/restaurant-setting';
import { tableSettingApi } from '@/api/table-setting';
import { sort } from '@/helpers/iteration';
import type { FdoLayout, FdoLayoutSetting, FdoTable } from '@feedmepos/core/entity';
import { defineStore } from 'pinia';


interface TableSettingState {
  layout: FdoLayoutSetting;
  tables: FdoTable[];
}

interface GroupedTables extends Omit<FdoLayout, 'tables'> {
  tables: FdoTable[];
}

function initTableSettingState(): TableSettingState {
  return {
    layout: {
      _id: '',
      // _rev: '',
      layouts: [],
    },
    tables: [],
  };
}

export const useTableSettingStore = defineStore('tableSetting', {
  state: () => {
    return initTableSettingState();
  },
  getters: {
    sortedTables: (state): FdoTable[] => {
      return sort(state.tables, { selector: 'name' });
    },
    sortedLayouts: (state): FdoLayout[] => {
      const sortByName = state.layout.layouts.some((layout) => layout.position === undefined);
      if (sortByName) {
        return sort(state.layout.layouts, { selector: 'name' });
      }

      return state.layout.layouts.sort((a, b) => a.position - b.position);
    },
    groupedTables(): GroupedTables[] {
      const withinLayouts = this.sortedLayouts.map((layout) => {
        return {
          ...layout,
          tables: this.sortedTables.filter((table) => layout.tables.some((v) => v.id === table._id)),
        };
      });
      const tables = this.layout.layouts.flatMap((v) => v.tables).map((v) => v.id);
      const otherTables = this.sortedTables.filter((t) => !tables.includes(t._id));
      if (!otherTables.length) return withinLayouts;
      return [
        ...withinLayouts,
        {
          id: '',
          name: 'Other',
          position: withinLayouts.length,
          tables: otherTables,
        },
      ];
    },
  },
  actions: {
    async readTables(foodCourtId?: string) {
      const resp = await tableSettingApi.readTables();
      
      const settings = await restaurantApi.getMobileOrderSetting();
      if (foodCourtId != undefined){
        this.tables = resp.map(e => ({ ...e, qrLink: `${appEndpoint.customerV3App}/dine?seat=${e.name}&fci=${foodCourtId}` }));
      }
      else if (settings.enableV3) {
        this.tables = [
          ...resp,
        ];
      }
      else this.tables = resp;
    },
    async createTables(layout: FdoLayout | undefined, tables: string[]) {
      await tableSettingApi.createTables(tables.map((t) => ({
        name: t.trim(),
      })));
      await this.readTables();
      if (layout) {
        const tableIds = this.tables
          .filter((t) => tables.includes(t.name))
          .map((t) => t._id);
        await this.moveTables('', layout.id, tableIds);
      }
    },
    async updateTable(table: FdoTable) {
      await tableSettingApi.updateTable(table);
      await this.readTables();
    },
    async deleteTables(tableIds: string[]) {
      this.tables = await tableSettingApi.deleteTables(tableIds);
    },
    async readLayouts() {
      this.layout = await tableSettingApi.readLayouts();
    },
    async createLayout(name: string) {
      this.layout = await tableSettingApi.createLayout({
        layoutSetting: this.layout,
        payload: {
          id: '',
          name,
          position: this.layout.layouts.length,
          tables: [],
        },
      });
    },
    async updateLayout(layout: FdoLayout) {
      this.layout = await tableSettingApi.updateLayout({
        ...this.layout,
        layouts: this.layout.layouts.map((l) => l.id === layout.id ? layout : l),
      });
    },
    async deleteLayout(layout: FdoLayout) {
      this.layout = await tableSettingApi.updateLayout({
        ...this.layout,
        layouts: this.layout.layouts.filter((l) => l.id !== layout.id),
      });
    },
    async moveTables(fromLayout: string, toLayout: string, tableIds: string[]) {
      const layouts = this.layout.layouts.map(l => {
        if (fromLayout === l.id) {
          return {
            ...l,
            tables: l.tables.filter((t) => !tableIds.includes(t.id))
          };
        }

        if (toLayout === l.id) {
          return {
            ...l,
            tables: [
              ...l.tables,
              ...tableIds.map((id) => ({ id })),
            ],
          };
        }

        return l;
      });
      this.layout = await tableSettingApi.updateLayout({
        ...this.layout,
        layouts,
      });
    },
    async updateSetting(is: boolean) {
      await tableSettingApi.updateSetting(is);
    }
  },
});
